@@ -5,9 +5,17 @@ test_that("RAVE preprocess pipeline 2.0", {
   data_dir <- tempfile()
   dir_create2(data_dir)
 
+  data_dir_old <- raveio::raveio_getopt("data_dir")
+  file_structure_old <- raveio::raveio_getopt("file_structure")
+  max_worker_old <- raveio::raveio_getopt("max_worker")
   on.exit({
     unlink(data_dir, recursive = TRUE)
+
     load_setting(reset_temp = TRUE)
+    raveio::raveio_setopt('data_dir', data_dir_old)
+    raveio::raveio_setopt('file_structure', file_structure_old)
+    raveio::raveio_setopt('max_worker', max_worker_old)
+
   })
 
   raveio::raveio_setopt('data_dir', data_dir, .save = FALSE)
